; pictock.asm
; PIC-Tock: video clock
;
; Copyright 1994, 1995 Eric Smith
;
; pictock.asm is free software; you can redistribute it and/or modify it under
; the terms of the GNU General Public License version 2 as published by the
; Free Software Foundation.  Note that I am not granting permission to
; redistribute or modify pictock.asm under the terms of any later version of
; the General Public License.
;
; This program is distributed in the hope that it will be useful (or at least
; amusing), but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License along with
; this program (in the file "COPYING"); if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; $Header: /usr/home/kolwynia/eric/pic/pictock/RCS/pictock.asm,v 1.9 1995/06/06 20:03:37 eric Exp eric $


; The PIC is clocked at 18.432 MHz, for a cycle time of 217 nS.
;
;               cycles  description             equation        calc'ed value
;------------------------------------------------------------------------------
;H      equ     300     ; horizontal total                      65.1 uS
;HSYNC  equ     23      ; horizontal sync       .075 * H        5.0 uS
;FP     equ     6       ; front porch           .02 * H min     1.3 uS
;BP     equ     21      ; back porch            .07 * H         4.6 uS
;HBLANK equ     HSYNC+FP+BP     ; horizontal blanking
;HACT   equ     H-HBLANK        ; horizontal active

;EP     equ     9       ; equalizing pulse      .04 * H         2.542225US
;VP     equ     98      ; vertical pulse        .43 * H         27.328892US
;EP2VP  equ     105     ; equal to vert pulses  .46 * H         29.235559US
;VP2EP  equ     16      ; vert to equal pulses  .07 * H         4.448894US

;               lines   description             equation        calc'ed value
;------------------------------------------------------------------------------
;V      equ     256     ; vertical frame        256 * H         16.67 mS
;VBLANK equ     20      ; vertical blank                
;VACT   equ     V-VBLANK ; vertical active      V - VBLANK      236 lines
;VEQ    equ     3       ; vertical equalizing   -               3 *H
;VSER   equ     3       ; vertical serration    -               3 *H

	device  PIC16C71,HS_OSC,WDT_OFF,PROTECT_OFF

; for instruction destination argument
f	equ     1
w	equ     0

ind	equ     0               ; used for indirect via fsr
rtcc	equ     1               ; real time clock/counter
pcl	equ     2               ; program counter

status	equ     3               ; status register:
cf      equ     0               ;  bit 0 = carry flag
dcf     equ     1               ;  bit 1 = digit carry flag
zf      equ     2               ;  bit 2 = zero flag
pdf     equ     3               ;  bit 3 = power down flag
tof     equ     4               ;  bit 4 = time out flag
				;  bits 5,6,7 do not apply to PIC16C54

fsr	equ     4               ; file select register (index register)

porta   equ     5               ; I/O port A:
pzt     equ     2               ;  bit 2 = beeper
bmode	equ	1		;  bit 1 = mode set button
badj	equ	0		;  bit 0 = adjust button

bmask	equ	3


portb   equ     6               ; I/O port B:
portv   equ     portb
video   equ     0               ;  bit 0 = video (active high)
blank   equ     5               ;  bit 5 = blanking (active low)
clamp   equ     6               ;  bit 6 = sync clamp (active low)

;          pb6     pb5     pb0
;         clamp   blank   video     voltage
;-----------------------------------------------------------------------------
; sync      0       x       x       0.00 V
; blank     1       0       0       0.30 V
; black     1       1       0       0.35 V
; white     1       1       1       1.05 V

pclath	equ     0ah
intcon	equ     0bh

adcon1	equ	88h

vsize   equ     192             ; playfield height in scan lines

ram     equ     0ch             ; start of RAM
body    equ     000h            ; beginning of program

	org     ram             ; start of RAM

vline   ds      1               ; current scan line
vflags	ds	1		; video flags:
fvblank	equ	0		;   video blanking
fvsync	equ	1		;   video sync

sndcnt  ds      1               ; sound count (fields)
sndmsk  ds      1               ; sound mask

snd4k   equ     002h            ; 3.8 kHz
snd2k   equ     004h            ; 1.9 kHz
snd1k   equ     008h            ; 960 Hz
snd480  equ     010h            ; 480 Hz
snd240  equ     020h            ; 240 Hz
snd120  equ     040h            ; 120 Hz

charrow ds      1               ; character row number 0..6
shift	ds	1

temp    ds      1               ; very temporary
random  ds      1               ; "random" number, incremented while waiting
				;   for game to start

msd	ds	1		; temps for binary to BCD conversion
lsd	ds	1

bstate	ds	1		; button state
bcount	ds	1		; button state frame count

field	ds	1
second	ds	1
minute	ds	1
hour	ds	1
day	ds	1
month	ds	1
year	ds	1


msgbeg	ds	1		; index into msgtbl of start of currently
				;   displayed message
msgend	ds	1		; index into msgtbl of end of currently
				;   displayed message
msgidx	ds	1		; index into msgtbl of first displayed char
				;   of currently displayed message
msgshift ds	1		; smooth scroll pixel count


msglen	equ	9
msg	ds	10	; msglen


	org     body            ; start of ROM

	goto    reset           ; reset vector
	nop
	nop
	nop
	goto    reset           ; interrupt vector      

; tables are in page zero so we don't have to mess with pclath

deln	addwf	pcl
delay21 nop
delay20 nop
delay19	nop
delay18 nop
delay17	nop
delay16 nop
delay15	nop
delay14	nop
delay13 nop
delay12	nop
delay11 nop
delay10	nop
delay9  nop
delay8  nop
delay7  nop
delay6  nop
delay5  nop
delay4  return

linetb8:
	addwf	pcl		; 038
	retlw	miscl		; 040  lines 0-7    \
	retlw	blackl		; 040  lines 8-15    > (lines 0-20 blanking)
	retlw	blackl		; 040  lines 16-23  /
	retlw	whitel		; 040  lines 24-31    white line
	retlw	blackl		; 040  lines 32-39
	retlw	blackl		; 040  lines 40-47
	retlw	blackl		; 040  lines 48-55
	retlw	blackl		; 040  lines 56-63

	retlw	message		; 040  lines 64-71
	retlw	message		; 040  lines 72-79
	retlw	message		; 040  lines 80-87
	retlw	message		; 040  lines 88-95

	retlw	blackl		; 040  lines 96-103
	retlw	blackl		; 040  lines 104-111
	retlw	blackl		; 040  lines 112-119
	retlw	miscl2		; 040  lines 120-127

	retlw	message		; 040  lines 128-135
	retlw	message		; 040  lines 136-143
	retlw	message		; 040  lines 144-151
	retlw	message		; 040  lines 152-159

	retlw	blackl		; 040  lines 160-167
	retlw	blackl		; 040  lines 168-175
	retlw	blackl		; 040  lines 176-183

	retlw	miscl3		; 040  lines 184-191

	retlw	message		; 040  lines 192-199
	retlw	message		; 040  lines 200-207
	retlw	message		; 040  lines 208-215
	retlw	message		; 040  lines 216-223

;	retlw	blackl		; 040  lines 192-199
;	retlw	blackl		; 040  lines 200-207
;	retlw	blackl		; 040  lines 208-215
;	retlw	blackl		; 040  lines 216-223

	retlw	blackl		; 040  lines 224-231
	retlw	blackl		; 040  lines 232-239
	retlw	whitel		; 040  lines 240-247  white line
	retlw	blackl		; 040  lines 248-255

linetb1:
	addwf	pcl
	goto	dotick
	goto	conv1
	goto	conv2
	goto	conv3
	goto	blackl
	goto	blackl
	goto	blackl
	goto	blackl

linetb2:
	addwf	pcl
	goto	debounce
	goto	conv4
	goto	conv5
	goto	conv6
	goto	blackl
	goto	blackl
	goto	blackl
	goto	blackl

linetb3:
	addwf	pcl		; 057
	goto	msgsetup	; 059
	goto	blackl
	goto	blackl
	goto	blackl
	goto	blackl
	goto	blackl
	goto	blackl
	goto	blackl

hex2char:
	addwf	pcl
	retlw	ch0
	retlw	ch1
	retlw	ch2
	retlw	ch3
	retlw	ch4
	retlw	ch5
	retlw	ch6
	retlw	ch7
	retlw	ch8
	retlw	ch9
	retlw	chA
	retlw	chB
	retlw	chC
	retlw	chD
	retlw	chE
	retlw	chF

; end of tables


reset:
	movlw	0dfh		; enable timer
	option

	movlw   0fbh            ; set port A to all inputs except RA2 for
	tris	5		;   beeper

	movlw   080h            ; set port B to all outputs except RB7
	tris    6

	movlw	adcon1		; Microchip brain damage strikes again!
	movwf	fsr		; Port A defaults to A/D!!!
	movlw	03h
	movwf	ind

	movlw   0ffh            ; set up for line 0
	movwf   vline

	movlw	ttlbeg
	movwf	msgidx
	clrf	msgshift

	movlw   snd1k           ; beep 1 KHz for about 1/5 sec
	movwf   sndmsk
	movlw   12
	movwf   sndcnt

	call	clrtime
	goto	nextline

; resync to timer

syncup				; cycle  timer
	movf	rtcc,w		; 268    f8 f9 fa fb
	andlw	3		; 269    f9 fa fb fc
	addwf	pcl		; 270    fa fb fc fd
	nop			; 272    fc 
	nop			; 273    fd fd
	nop			; 274    fe fe fe
	nop			; 275    ff ff ff ff

synclp	movf	rtcc,w		; 276    00
	btfss	status,zf	; 277    01
	goto	synclp		; 278    02
	bcf     portv,video	; 279

; nextline is the entry point for the normal playfield display
nextline:
	bcf	porta,pzt	; 280 do sound - assume outputs low

	incf    vline           ; 281 advance to next line
	movf    vline,w         ; 282

	bsf     portv,clamp     ; 283 end vertical sync pulse

	andwf   sndmsk,w        ; 284 do sound
	btfss   status,zf       ; 285
	bsf     porta,pzt       ; 286

	clrf	vflags		; 287

	movf	vline,w		; 288 figure out whether we're in vblank time
	addlw	256-21		; 289
	btfss	status,cf	; 290
	bsf	vflags,fvblank	; 291

	movf	vline,w		; 292 figure out whether we're in vsync time
	addlw	256-3		; 293
	addlw	256-3		; 294
	btfss	status,cf	; 295
	bsf	vflags,fvsync	; 296

	goto	.+1		; 297
	nop			; 299

	bcf     portv,blank     ; 000 start horizontal blanking (front porch)
	call    delay5          ; 001
	bcf     portv,clamp     ; 006 end front porch, start horizontal sync

	movf	vline,w		; 007
	movwf	charrow		; 008
	rrf	charrow		; 009
	rrf	charrow		; 010
	movlw	7		; 011
	andwf	charrow		; 012

	call    delay4          ; 013
	clrf	rtcc		; 017
	call	delay10		; 018

	btfss	vflags,fvsync	; 028 if we're not in vsync time,
	bsf     portv,clamp     ; 029   end horizontal sync

; load address of line handler into W

	movf	vline,w		; 030
	movwf	temp		; 031
	rrf	temp		; 032
	rrf	temp		; 033
	rrf	temp,w		; 034
	andlw	01fh		; 035
	call	linetb8		; 036

	clrf	pclath		; 042

	call	delay6		; 043

	btfss	vflags,fvblank	; 049 if we're not in vblank time
	bsf     portv,blank     ; 050   end back porch, end horizontal blanking

	movwf	pcl		; 051 dispatch to line handler


whitel:	bsf     portv,video
blackl:	goto	syncup


miscl:	movf	vline,w
	andlw	7
	goto	linetb1


miscl2:	movf	vline,w		; 053
	andlw	7		; 054
	goto	linetb2		; 055


miscl3:	movf	vline,w		; 053
	andlw	7		; 054
	goto	linetb3		; 055


; NOTE: PCLATH must be zero on entry

message:
	movf	charrow,w
	xorlw	7
	btfsc	status,zf
	goto	syncup

	movf	shift,w
	call	deln

	movlw	msg
	movwf	fsr

	movlw	msglen
	movwf	temp

	movlw	3
	movwf	pclath

msglp:	movf	ind,w		; 00
	addwf	charrow,w	; 01
	call	chargen		; 02
	movwf	portv		; 08
	rrf	portv		; 09
	rrf	portv		; 10
	rrf	portv		; 11
	rrf	portv		; 12
	bcf	portv,video	; 13
	incf	fsr		; 14
	decfsz	temp		; 15
	goto	msglp		; 16-17

	clrf	pclath
	goto	syncup


;deln3   addlw   0ffh    ; subtract 1
;deln2   addlw   0ffh    ; subtract 1
;deln1   addlw   0ffh    ; subtract 1
;deln0   addlw   0fch    ; subtract 4
;delayl  addlw   0fch    ; subtract 4
;	btfss   status,zf
;	goto    delayl
;	return


; reset the time
clrtime
	clrf	field
	clrf	second
	clrf	minute
	clrf	hour
	clrf	day
	clrf	month
	clrf	year
	return


dotick:
	clrf	shift

	movf	sndcnt,w	; if (sndcnt) sndcnt--;
	btfss	status,zf
	decf	sndcnt

	movf	sndcnt,w	; if (! sndcnt) sndmsk = 0;
	btfsc	status,zf
	clrf	sndmsk

	incf	field
	movf	field,w
	xorlw	60
	btfss	status,zf
	goto	dt9
	clrf	field

	bcf	porta,pzt
	bsf	porta,pzt
	bcf	porta,pzt

;	movlw	snd1k
;	movwf	sndmsk
;	movlw	1
;	movwf	sndcnt

	incf	second
	movf	second,w
	xorlw	60
	btfss	status,zf
	goto	dt9
	clrf	second

	movlw	snd2k
	movwf	sndmsk
	movlw	1
	movwf	sndcnt

	incf	minute
	movf	minute,w
	xorlw	60
	btfss	status,zf
	goto	dt9
	clrf	minute

	incf	hour
	movf	hour,w
	xorlw	24
	btfss	status,zf
	goto	dt9
	clrf	hour

dt9:	goto	syncup


conv1:	movf	second,w
	call	bin2bcd
	movf	lsd,w
	call	hex2char
	movwf	msg+7
	movf	msd,w
	call	hex2char
	movwf	msg+6

;	movf	field,w
;	addlw	256-30
;	movlw	chspace
;	btfss	status,cf
	movlw	chcolon

	movwf	msg+5
	movwf	msg+2

	movlw	chspace
	movwf	msg+8
	movwf	msg+9

	goto	syncup


conv2:	movf	minute,w
	call	bin2bcd
	movf	lsd,w
	call	hex2char
	movwf	msg+4
	movf	msd,w
	call	hex2char
	movwf	msg+3

	goto	syncup


conv3:	movf	hour,w
	call	bin2bcd
	movf	lsd,w
	call	hex2char
	movwf	msg+1
	movf	msd,w
	call	hex2char
	movwf	msg+0

	goto	syncup


conv4:	movf	bstate,w
	call	bin2bcd
	movf	lsd,w
	call	hex2char
	movwf	msg+1
	movf	msd,w
	call	hex2char
	movwf	msg+0

	movlw	chspace
	movwf	msg+2
	movwf	msg+5
	movwf	msg+8

	goto	syncup


conv5:	movf	bcount,w
	call	bin2bcd
	movf	lsd,w
	call	hex2char
	movwf	msg+4
	movf	msd,w
	call	hex2char
	movwf	msg+3

	goto	syncup


conv6:	movf	porta,w
	andlw	0fh
	call	hex2char
	movwf	msg+7
	swapf	porta,w
	andlw	0fh
	call	hex2char
	movwf	msg+6

	goto	syncup


bin2bcd:
	clrf	msd
	movwf	lsd
b1:	movlw	10
	subwf	lsd,w
	btfss	status,cf
	goto	b9
	movwf	lsd
	incf	msd
	goto	b1
b9:	return


msgsetup:
	movlw	ttlbeg		; 061
	movwf	msgbeg		; 062
	movlw	ttlend		; 063
	movwf	msgend		; 064

	incf	msgshift	; 065
	movf	msgshift,w	; 066
	xorlw	18		; 067
	btfss	status,zf	; 068
	goto	ms1		; 069

	clrf	msgshift

	incf	msgidx
	movf	msgidx,w
	xorwf	msgend,w
	btfss	status,zf
	goto	ms1

	movf	msgbeg,w
	movwf	msgidx

ms1:	movf	msgshift,w	; 071
	movwf	shift

msgcopy:
	movf	msgidx,w
	movwf	temp

	movlw	msg
	movwf	fsr

	movlw	2
	movwf	pclath

msgc0:	movf	temp,w		; 0
	call	msgtbl		; 1
	movwf	ind		; 7

	incf	fsr		; 8
	movf	fsr,w		; 9
	xorlw	msg+msglen	; 10
	btfsc	status,zf	; 11
	goto	msgc9		; 12

	incf	temp		; 13
	goto	msgc0		; 14-15

	movf	msgbeg,w	; 18
	movwf	temp		; 19
	goto	msgc0		; 20-21

msgc9:	clrf	pclath
	goto	syncup


debounce:
	movf	porta,w		; get the current button state
	andlw	bmask
	xorlw	bmask
	movwf	temp

	xorlw	bmask		; are both buttons pressed?
	btfsc	status,zf
	goto	db8

	movf	temp,w		; did the buttons stay in the same state?
	xorlw	bstate
	btfss	status,zf
	goto	db8

db2:	movf	bcount,w	; buttons stayed the same, check count
	btfsc	status,zf
	goto	db9		; was already zero, don't do anything

	decf	bcount		; decrement count
	btfsc	status,zf
	goto	db9		; not yet zero, don't do anything

; MORE CODE NEEDED HERE (with apologies to Wayne Wall)
	goto	db9


db8:	movlw	99 ; 3		; start debounce timer over
	movwf	bcount
	goto	db9

db9:	movf	temp,w		; save button state
	movwf	bstate
	goto	syncup


	org	0200h

msgtbl:
	addwf	pcl
ttlbeg	equ	.-(msgtbl+1)
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chS
	retlw	chT
	retlw	chU
	retlw	chP
	retlw	chI
	retlw	chD
	retlw	chspace
	retlw	chP
	retlw	chI
	retlw	chC
	retlw	chspace
	retlw	chT
	retlw	chR
	retlw	chI
	retlw	chC
	retlw	chK
	retlw	chspace
	retlw	choctothorpe
	retlw	ch3
	retlw	ch7
	retlw	chcolon
	retlw	chspace
	retlw	chP
	retlw	chI
	retlw	chC
	retlw	chdash
	retlw	chT
	retlw	chO
	retlw	chC
	retlw	chK
	retlw	chleftparen
	retlw	chT
	retlw	chM
	retlw	chrightparen
	retlw	chspace
	retlw	chspace
	retlw	chS
	retlw	chO
	retlw	chF
	retlw	chT
	retlw	chW
	retlw	chA
	retlw	chR
	retlw	chE
	retlw	chcolon
	retlw	chspace
	retlw	chE
	retlw	chR
	retlw	chI
	retlw	chC
	retlw	chspace
	retlw	chS
	retlw	chM
	retlw	chI
	retlw	chT
	retlw	chH
	retlw	chspace
	retlw	chspace
	retlw	chH
	retlw	chA
	retlw	chR
	retlw	chD
	retlw	chW
	retlw	chA
	retlw	chR
	retlw	chE
	retlw	chcolon
	retlw	chspace
	retlw	chR
	retlw	chI
	retlw	chC
	retlw	chH
	retlw	chA
	retlw	chR
	retlw	chD
	retlw	chspace
	retlw	chO
	retlw	chT
	retlw	chT
	retlw	chO
	retlw	chS
	retlw	chE
	retlw	chN
	retlw	chspace
	retlw	chspace
	retlw	chM
	retlw	chO
	retlw	chR
	retlw	chA
	retlw	chL
	retlw	chspace
	retlw	chS
	retlw	chU
	retlw	chP
	retlw	chP
	retlw	chO
	retlw	chR
	retlw	chT
	retlw	chcolon
	retlw	chspace
	retlw	chG
	retlw	chA
	retlw	chR
	retlw	chY
	retlw	chspace
	retlw	chL
	retlw	chA
	retlw	chW
	retlw	chR
	retlw	chE
	retlw	chN
	retlw	chC
	retlw	chE
	retlw	chspace
	retlw	chspace
	retlw	chB
	retlw	chR
	retlw	chA
	retlw	chI
	retlw	chN
	retlw	chS
	retlw	chcolon
	retlw	chspace
	retlw	chM
	retlw	chI
	retlw	chC
	retlw	chR
	retlw	chO
	retlw	chC
	retlw	chH
	retlw	chI
	retlw	chP
	retlw	chspace
	retlw	chP
	retlw	chI
	retlw	chC
	retlw	ch1
	retlw	ch6
	retlw	chC
	retlw	ch7
	retlw	ch1
ttlend	equ	.-(msgtbl+1)
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chspace
	retlw	chspace

	org     0300h

	opt	nol

p00000	equ	0e0h
p00001	equ	0f0h
p00010	equ	0e8h
p00011	equ	0f8h
p00100	equ	0e4h
p00101	equ	0f4h
p00110	equ	0ech
p00111	equ	0fch
p01000	equ	0e2h
p01001	equ	0f2h
p01010	equ	0eah
p01011	equ	0fah
p01100	equ	0e6h
p01101	equ	0f6h
p01110	equ	0eeh
p01111	equ	0feh
p10000	equ	0e1h
p10001	equ	0f1h
p10010	equ	0e9h
p10011	equ	0f9h
p10100	equ	0e5h
p10101	equ	0f5h
p10110	equ	0edh
p10111	equ	0fdh
p11000	equ	0e3h
p11001	equ	0f3h
p11010	equ	0ebh
p11011	equ	0fbh
p11100	equ	0e7h
p11101	equ	0f7h
p11110	equ	0efh
p11111	equ	0ffh


chargen
	addwf	pcl
chbase	equ	$
ch0	equ	$-chbase
chO	equ	$-chbase
	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p01110	; .***.
ch1	equ	$-chbase
	retlw	p00100	; ..*..
	retlw	p01100	; .**..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p11111	; *****
ch2	equ	$-chbase
	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p00001	; ....*
	retlw	p00010	; ...*.
	retlw	p00100	; ..*..
	retlw	p01000	; .*...
	retlw	p11111	; *****
ch3	equ	$-chbase
	retlw	p11111	; *****
	retlw	p00010	; ...*.
	retlw	p00100	; ..*..
	retlw	p01110	; .***.
	retlw	p00001	; ....*
	retlw	p10001	; *...*
	retlw	p01110	; .***.
ch4	equ	$-chbase
	retlw	p00010	; ...*.
	retlw	p00110	; ..**.
	retlw	p01010	; .*.*.
	retlw	p10010	; *..*.
	retlw	p11111	; *****
	retlw	p00010	; ...*.
	retlw	p00010	; ...*.
ch5	equ	$-chbase
	retlw	p11111	; *****
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p11110	; ****.
	retlw	p00001	; ....*
	retlw	p10001	; *...*
	retlw	p01110	; .***.
ch6	equ	$-chbase
	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10000	; *....
	retlw	p11110	; ****.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p01110	; .***.
ch7	equ	$-chbase
	retlw	p11111	; *****
	retlw	p00001	; ....*
	retlw	p00001	; ....*
	retlw	p00010	; ...*.
	retlw	p00010	; ...*.
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
ch8	equ	$-chbase
 	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
 	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
 	retlw	p01110	; .***.
ch9	equ	$-chbase
 	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
 	retlw	p01110	; .***.
	retlw	p00001	; ....*
	retlw	p00010	; ...*.
chA	equ	$-chbase
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p01010	; .*.*.
	retlw	p10001	; *...*
	retlw	p11111	; *****
chH	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p11111	; *****
chY	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p01010	; .*.*.
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
chR	equ	$-chbase
	retlw	p11110	; ****.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p11110	; ****.
	retlw	p10100	; *.*..
	retlw	p10010	; *..*.
chK	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p10010	; *..*.
	retlw	p10100	; *.*..
	retlw	p11000	; **...
	retlw	p10100	; *.*..
	retlw	p10010	; *..*.
chM	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p11011	; **.**
	retlw	p10101	; *.*.*
	retlw	p10101	; *.*.*
	retlw	p10001	; *...*
chN	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p11001	; **..*
	retlw	p10101	; *.*.*
	retlw	p10011	; *..**
chW	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10101	; *.*.*
	retlw	p10101	; *.*.*
	retlw	p11011	; **.**
chU	equ	$-chbase
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
chC	equ	$-chbase
	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10001	; *...*
chG	equ	$-chbase
	retlw	p01110	; .***.
	retlw	p10001	; *...*
	retlw	p10000	; *....
	retlw	p10111	; *.***
	retlw	p10001	; *...*
	retlw	p10001	; *...*
chS	equ	$-chbase
	retlw	p01111	; .****
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p01110	; .***.
	retlw	p00001	; ....*
	retlw	p00001	; ....*
chD	equ	$-chbase
	retlw	p11110	; ****.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p10001	; *...*
chB	equ	$-chbase
	retlw	p11110	; ****.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
chP	equ	$-chbase
	retlw	p11110	; ****.
	retlw	p10001	; *...*
	retlw	p10001	; *...*
	retlw	p11110	; ****.
chL	equ	$-chbase
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10000	; *....
chE	equ	$-chbase
	retlw	p11111	; *****
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p11110	; ****.
	retlw	p10000	; *....
	retlw	p10000	; *....
chI	equ	$-chbase
	retlw	p11111	; *****
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
chT	equ	$-chbase
	retlw	p11111	; *****
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
	retlw	p00100	; ..*..
chF	equ	$-chbase
	retlw	p11111	; *****
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p11110	; ****.
	retlw	p10000	; *....
	retlw	p10000	; *....
	retlw	p10000	; *....
chdash	equ	$-chbase
	retlw	p00000	; .....
	retlw	p00000	; .....
	retlw	p00000	; .....
	retlw	p11111	; *****
chspace	equ	$-chbase
	retlw	p00000	; .....
	retlw	p00000	; .....
	retlw	p00000	; .....
	retlw	p00000	; .....
	retlw	p00000	; .....
	retlw	p00000	; .....
chcolon	equ	$-chbase
	retlw	p00000	; .....
	retlw	p00110	; ..**.
	retlw	p00110	; ..**.
	retlw	p00000	; .....
	retlw	p00110	; ..**.
	retlw	p00110	; ..**.
	retlw	p00000	; .....
chand	equ	$-chbase
	retlw	p01100	; .**..
	retlw	p10010	; *..*.
	retlw	p10100	; *.*..
	retlw	p01000	; .*...
	retlw	p10101	; *.*.*
	retlw	p10010	; *..*.
	retlw	p01101	; .**.*
choctothorpe equ $-chbase
	retlw	p01010	; .*.*.
	retlw	p01010	; .*.*.
	retlw	p11111	; *****
	retlw	p01010	; .*.*.
	retlw	p11111	; *****
	retlw	p01010	; .*.*.
	retlw	p01010	; .*.*.
chleftparen equ $-chbase
	retlw	p00010	; ...*.
	retlw	p00100	; ..*..
	retlw	p01000	; .*...
	retlw	p01000	; .*...
	retlw	p01000	; .*...
	retlw	p00100	; ..*..
	retlw	p00010	; ...*.
chrightparen equ $-chbase
	retlw	p01000	; .*...
	retlw	p00100	; ..*..
	retlw	p00010	; ...*.
	retlw	p00010	; ...*.
	retlw	p00010	; ...*.
	retlw	p00100	; ..*..
	retlw	p01000	; .*...
	opt	l

	end
